//
//  ImageDrawView.m
//  FlipLab
//
//  Created by Paul Kim on 1/30/09.
//  Copyright 2009 Noodlesoft, LLC. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
#import "ImageDrawView.h"
#import "NSImage+FlippedDrawing.h"

@implementation ImageDrawView

@synthesize drawMethod;
@synthesize isDestinationFlipped;
@synthesize isImageFlipped;
@synthesize scale;

#define DEFAULT_IMAGE_SIZE		100.0
#define DEFAULT_COMPOSITE_POINT	40.0

/*
 * Renders the sample image. Takes the flipping into account. Normally, the image is an arrow pointing up
 * with the text "UP". If drawn into a flipped coordinate system, the image is an arrow pointing down with
 * the word "DOWN".
 */
- (void)drawIt:(id)rep
{
	NSBezierPath		*boundsPath, *path;
	NSSize				size;
	NSRect				rect;
	CGFloat				margin;
	NSString			*text;
	NSDictionary		*attributes = [NSDictionary dictionaryWithObjectsAndKeys:
									   [NSFont fontWithName:@"Helvetica Bold" size:24], NSFontAttributeName, nil];
	
	size = [rep size];
	rect = NSMakeRect(0, 0, size.width, size.height);
	
	// Change text to indicate orientation based on flippedness
	if ([[NSGraphicsContext currentContext] isFlipped])
	{
		text = @"DOWN";
	}
	else
	{
		text = @"UP";
	}
	
	boundsPath = [NSBezierPath bezierPathWithRect:rect];
	[[NSColor colorWithCalibratedRed:0.835 green:0.933 blue:0.776 alpha:1.0] set];
	[boundsPath fill];
	
	// Draw the arrow. Will draw up unless things are flipped in which case things are drawn down.
	margin = 0.10 * NSHeight(rect);
	path = [NSBezierPath bezierPath];
	[path moveToPoint:NSMakePoint(NSMidX(rect), NSMaxY(rect) - margin)];
	[path lineToPoint:NSMakePoint(NSMaxX(rect) - 2 * margin, NSMaxY(rect) - 3.5 * margin)];
	[path lineToPoint:NSMakePoint(NSMaxX(rect) - 3.5 * margin, NSMaxY(rect) - 3.5 * margin)];
	[path lineToPoint:NSMakePoint(NSMaxX(rect) - 3.5 * margin, NSMinY(rect) + 3 * margin)];
	[path lineToPoint:NSMakePoint(NSMinX(rect) + 3.5 * margin, NSMinY(rect) + 3 * margin)];
	[path lineToPoint:NSMakePoint(NSMinX(rect) + 3.5 * margin, NSMaxY(rect) - 3.5 * margin)];
	[path lineToPoint:NSMakePoint(NSMinX(rect) + 2 * margin, NSMaxY(rect) - 3.5 * margin)];
	[path closePath];
	
	[[NSColor colorWithCalibratedRed:0.318 green:0.745 blue:0.933 alpha:1.0] set];
	[path fill];
	[[NSColor blackColor] set];
	[path stroke];
	
	// Draw a little blip at the origin
	[[NSColor redColor] set];
	[[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(NSMinX(rect) - 5.0, NSMinY(rect) - 5.0, 10.0, 10.0)] fill];
	
	// Note that no adjustments are made to the coordinates when flipped. In addition, the text will appear right-side-up
	// regardless of whether the coordinate system is flipped or not.
	size = [text sizeWithAttributes:attributes];
	rect = NSMakeRect(NSMinX(rect) + (NSWidth(rect) - size.width) / 2, NSMinY(rect) + 0.2 * margin, size.width, size.height);
	[[NSColor blackColor] set];
	[text drawInRect:rect withAttributes:attributes];
	
	[boundsPath setLineWidth:2.0];
	[[NSColor blackColor] set];
	[boundsPath stroke];
}

- (void)_regenerateImage
{	
	NSCustomImageRep *rep;
	
	[image release];
	image = [[NSImage alloc] initWithSize:NSMakeSize(DEFAULT_IMAGE_SIZE, DEFAULT_IMAGE_SIZE)];
	
	[image setFlipped:isImageFlipped];
	rep = [[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawIt:) delegate:self];
	[image addRepresentation:rep];
	[rep release];
}

- (void)awakeFromNib
{
	[self addObserver:self forKeyPath:@"drawMethod" options:0 context:nil];
	[self addObserver:self forKeyPath:@"destinationFlipped" options:0 context:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
					  ofObject:(id)object
                        change:(NSDictionary *)change
                       context:(void *)context
{
	[self setNeedsDisplay:YES];
}

- (void)setScale:(CGFloat)newScale
{
	NSRect		rect;
	
	scale = newScale;
	
	rect = [self frame];
	rect.size.width /= scale;
	rect.size.height /= scale;
	
	[self setBoundsSize:rect.size];
	[self setNeedsDisplay:YES];
}

- (BOOL)isFlipped
{
	return isDestinationFlipped;
}

- (void)setImageFlipped:(BOOL)flag
{
	if (isImageFlipped != flag)
	{
		isImageFlipped = flag;
		[image setFlipped:isImageFlipped];
		[image recache];
		[self setNeedsDisplay:YES];
	}
}

- (void)drawRect:(NSRect)rect
{
	NSBezierPath		*path;
	NSPoint				point;
	NSSize				size;
	NSRect				bounds, originalBounds;
	
	if (image == nil)
	{
		[self _regenerateImage];
	}
	
	bounds = [self bounds];

	originalBounds = [self frame];
	originalBounds.origin.x = NSMinX(bounds);
	originalBounds.origin.y = NSMinY(bounds);
	
	point = NSMakePoint(DEFAULT_COMPOSITE_POINT, DEFAULT_COMPOSITE_POINT);
	
	path = [NSBezierPath bezierPathWithRect:bounds];
	[[NSColor whiteColor] set];
	[path fill];
	
	[[NSColor redColor] set];
	// Draw a line at the y-coord where the image is drawn.
	[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(bounds) + 0.5, point.y)
							  toPoint:NSMakePoint(NSMaxX(bounds) - 0.5, point.y)];
	
	// Draw a little blip at the origin
	[[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(NSMinX(bounds) - 5.0, NSMinY(bounds) - 5.0, 10.0, 10.0)] fill];
	
	switch (drawMethod)
	{
		case kUseComposite:
			size = [image size];
			[image compositeToPoint:point fromRect:NSMakeRect(0, 0, size.width, size.height)
						  operation:NSCompositeSourceOver fraction:1.0];
			break;
		case kUseDraw:
			[image drawAtPoint:point fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];
			break;
		case kUseCorrectedDraw:
			[image drawAdjustedAtPoint:point fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];
			break;
	}
	
	// Draw the "original" bounds to compare against current scale.
	[[NSColor grayColor] set];
	[NSBezierPath strokeRect:originalBounds];
	
	[[NSColor blackColor] set];
	[path stroke];
}

- (void)mouseDown:(NSEvent *)event
{
	NSPoint			point;
	NSSize			imageSize;
	NSImage			*dragImage;
	
	point = [self convertPointFromBase:[event locationInWindow]];
	
	imageSize = [image size];

	point.x -= imageSize.width / 2.0;
	if ([self isFlipped])
	{
		point.y += imageSize.height / 2.0;
	}
	else
	{
		point.y -= imageSize.height / 2.0;
	}
	
	if (drawMethod == kUseCorrectedDraw)
	{
		// Will return the original image if it is already not flipped
		dragImage = [image unflippedImage];
	}
	else
	{
		dragImage = image;
	}
	
	[self dragImage:dragImage at:point offset:NSZeroSize event:event pasteboard:nil source:self slideBack:NO];
}

@end
